/******************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        combinations.ms
Date:           11/5/09
Description:    This script will calculate combinations and permutations
******************************************************************************/

try (destroyDialog combinations_rollout) catch()


struct CombinationPermutation
(
    fn get_factoral base_number limit_descent =
    (
        local result = 1.0
        local iterations = 0
        
        if limit_descent > 0 then
            iterations = limit_descent - 1
        else
            iterations = base_number - 1
        
        for i = 0 to iterations do
            result *= (base_number - i)
            
        result --return value
    ), --end fn
    
    /**************************************************************************
    Returns the total number of combinations with repetition. NOTE:
    Combinations with Repetition = (object + draw -1)! / draw! x (object - 1)!
    **************************************************************************/
    fn get_combo_with_rep total_objects draw_objects =
    (
        local result = 0
        local numerator = 0
        local denominator = 0
        
        numerator = this.get_factoral (total_objects + draw_objects - 1) 0
        denominator = this.get_factoral draw_objects 0
        denominator *= this.get_factoral (total_objects - 1) 0
        result = ((numerator / denominator) as integer)
        
        result --return value
    ), --end fn
    
    /**************************************************************************
    Returns the total number of combinations without repetition. NOTE:
    Combinations without Repetition = object! / draw! x (object - draw)! OR object! (but only to draw num) / draw!
    **************************************************************************/
    fn get_combo_no_rep total_objects draw_objects =
    (
        local result = 0
        local numerator = 0
        local denominator = 0
        
        numerator = this.get_factoral total_objects draw_objects
        denominator = this.get_factoral draw_objects 0
        result = ((numerator / denominator) as integer)
        
        result --return value
    ), --end fn
    
    /**************************************************************************
    Returns the total number of permutations with repitition. NOTE:
    Permutations with Repetition = pow objects draw
    **************************************************************************/
    fn get_perm_with_rep total_objects draw_objects =
    (
        local result = ((pow total_objects draw_objects) as integer)
        
        result --return value
    ), --end fn
    
    /**************************************************************************
    Returns the total number of permutations without repitition. NOTE:
    Permutations without Repetition = object! / (object - draw)! OR object! (but only to draw num, ie for 3 of 16 = 16 x 15 x 14)
    **************************************************************************/
    fn get_perm_no_rep total_objects draw_objects =
    (
        local result = ((this.get_factoral total_objects draw_objects) as integer)
        
        result --return value
    ) --end fn
) --end struct


( --put script into local scope
    global combinations_rollout

    rollout combinations_rollout "Combinations"
    (
        local combos = CombinationPermutation()
        
        group "Parameters"
        (
            spinner object_spinner "# of Objects:" range:[1,99,1] type:#integer scale:1 fieldWidth:50 align:#left
            spinner draw_spinner "# to Draw:    " range:[1,99,1] type:#integer scale:1 fieldWidth:50 align:#left
        ) --end group
        
        group "Combinations"
        (
            label combo_rep_label "Repitition:" align:#left across:2
            label combo_rep_result_label "1" align:#right
            label combo_no_rep_label "No Repitition:" align:#left across:2
            label combo_no_rep_result_label "1" align:#right
        ) --end group
        
        group "Permutations"
        (
            label perm_rep_label "Repitition:" align:#left across:2
            label perm_rep_result_label "1" align:#right
            label perm_no_rep_label "No Repitition:" align:#left across:2
            label perm_no_rep_result_label "1" align:#right
        ) --end group
        
        /**********************************************************************
        Object spinner event handler. Recalculates the results
        **********************************************************************/
        on object_spinner changed theValue do
        (
            if object_spinner.value < draw_spinner.value then
                object_spinner.value = draw_spinner.value
            else
            (
                result = combos.get_combo_with_rep object_spinner.value draw_spinner.value
                combo_rep_result_label.text = result as string
                result = combos.get_combo_no_rep object_spinner.value draw_spinner.value
                combo_no_rep_result_label.text = result as string
                result = combos.get_perm_with_rep object_spinner.value draw_spinner.value
                perm_rep_result_label.text = result as string
                result = combos.get_perm_no_rep object_spinner.value draw_spinner.value
                perm_no_rep_result_label.text = result as string
            ) --end else
        ) --end on
        
        /**********************************************************************
        Draw spinner event handler. Recalculates the results
        **********************************************************************/
        on draw_spinner changed theValue do
        (
            if draw_spinner.value > object_spinner.value then
                draw_spinner.value = object_spinner.value
            else
            (
                result = combos.get_combo_with_rep object_spinner.value draw_spinner.value
                combo_rep_result_label.text = result as string
                result = combos.get_combo_no_rep object_spinner.value draw_spinner.value
                combo_no_rep_result_label.text = result as string
                result = combos.get_perm_with_rep object_spinner.value draw_spinner.value
                perm_rep_result_label.text = result as string
                result = combos.get_perm_no_rep object_spinner.value draw_spinner.value
                perm_no_rep_result_label.text = result as string
            ) --end fn
        ) --end on
    ) --end rollout
    createDialog combinations_rollout 160 195
) --end local scope
